@@ -1,10 +1,13 @@
+use duplicate::duplicate_item;
 use filemagic::Magic;
+use walkdir::DirEntry;
 use yara;
 use anyhow::{Result, anyhow};
 use crate::filescanner::*;
 use crate::scanner_result;
 use crate::scanner_result::*;
 use std::cell::RefCell;
+use std::collections::HashMap;
 use std::io::Read;
 use std::path::Path;
 use walkdir::WalkDir;
@@ -15,6 +18,9 @@ use flate2::read::GzDecoder;
 use xz::read::XzDecoder;
 use bzip2::read::BzDecoder;
 
+#[cfg(target_family="unix")]
+use file_owner::PathExt;
+
 pub struct YaraFinding {
     pub identifier: String,
     pub namespace: String,
@@ -33,8 +39,57 @@ impl From<&yara::Rule<'_>> for YaraFinding {
     }
 }
 
+#[derive(Default)]
+struct YaraExternals {
+    filename: Option<String>,
+    filepath: Option<String>,
+    extension: Option<String>,
+    filetype: Option<String>,
+    md5: Option<String>,
+    owner: Option<String>
+}
+
+impl YaraExternals {
+    pub fn to_hashmap(&self) -> HashMap<&str, &str> {
+        let mut res = HashMap::new();
+
+        if let Some(x) = &self.filename  { res.insert("filename",  (&x).as_str());}
+        if let Some(x) = &self.filepath  { res.insert("filepath",  (&x).as_str());}
+        if let Some(x) = &self.extension { res.insert("extension", (&x).as_str());}
+        if let Some(x) = &self.filetype  { res.insert("filetype",  (&x).as_str());}
+        if let Some(x) = &self.md5       { res.insert("md5",       (&x).as_str());}
+        if let Some(x) = &self.owner     { res.insert("owner",     (&x).as_str());}
+
+        res
+    }
+
+    #[duplicate_item (
+        method_name      variable_name;
+        [with_filename]  [filename];
+        [with_filepath]  [filepath];
+        [with_extension] [extension];
+        [with_filetype]  [filetype];
+        [with_md5]       [md5];
+        [with_owner]     [owner]
+    )]
+    pub fn method_name(mut self, variable_name: String) -> Self {
+        self.variable_name = Some(variable_name);
+        self
+    }
+
+    pub fn dummy() -> Self {
+        Self::default()
+            .with_filename("-".to_owned())
+            .with_filepath("-".to_owned())
+            .with_extension("-".to_owned())
+            .with_filetype("-".to_owned())
+            .with_md5("-".to_owned())
+            .with_owner("dummy".to_owned())
+    }
+}
+
 pub struct YaraScanner {
-    rules: Vec<yara::Rules>,
+    rules: yara::Rules,
     scan_compressed: bool,
     magic: Magic,
     buffer: RefCell<Vec<u8>>
@@ -49,23 +104,47 @@ enum CompressionType {
 
 impl FileScanner for YaraScanner
 {
-    fn scan_file(&self, file: &Path) -> Vec<anyhow::Result<ScannerFinding>> {
+    fn scan_file(&self, file: &DirEntry) -> Vec<anyhow::Result<ScannerFinding>> {
         let mut results = Vec::new();
+        let file = file.path();
+        
+        let magic = match self.magic.file(file) {
+            Ok(magic) => {
+                log::info!("treating '{}' as '{}'", file.display(), &magic);
+                Some(magic)
+            }
+            Err(why) => { 
+                log::warn!("unable to determine file type for '{}': {}",
+                    file.display(), why);
+                None
+            }
+        };
+
+        // prepare externals, which are required by some signature-base rules
+        let mut externals = YaraExternals::default()
+            .with_filepath(file.display().to_string())
+            .with_filename(file.file_name().unwrap().to_str().unwrap().to_string())
+            .with_extension(match file.extension(){
+                Some(f) => f.to_string_lossy().to_string(),
+                None => "-".to_owned()
+            })
+            .with_filetype(magic.clone().or(Some("-".to_owned())).unwrap());
+        
+        externals = if cfg!(target_family = "unix") {
+            externals.with_owner(match file.display().to_string().owner(){
+                Ok(owner) => match owner.name() {
+                    Ok(name) => name.or(Some(owner.id().to_string())).unwrap(),
+                    Err(why) => return vec![Err(anyhow!(why))]
+                }
+                Err(why) => return vec![Err(anyhow!(why))]
+            })
+        } else {
+            externals.with_owner("-".to_owned())
+        };
 
         // check if the file is a compressed file
         let compression_type = 
         if self.scan_compressed {
-            let magic = match self.magic.file(file) {
-                Ok(magic) => {
-                    log::info!("treating '{}' as '{}'", file.display(), &magic);
-                    Some(magic)
-                }
-                Err(why) => { 
-                    log::warn!("unable to determine file type for '{}': {}",
-                        file.display(), why);
-                    None
-                }
-            };
 
             if let Some(m) = &magic {
                 if m == "XZ compressed data"                    {CompressionType::XZ}
@@ -101,24 +180,34 @@ impl FileScanner for YaraScanner
             }
         }
 
-        for rules in self.rules.iter() {
-            let scan_result = match self.buffer.borrow().is_empty() {
-                true => rules.scan_file(&file, 120),
-                false => rules.scan_mem(&self.buffer.borrow(), 120).or_else(|e| Err(yara::Error::Yara(e))),
-            };
+        let mut scanner = match self.rules.scanner() {
+            Err(why) => return vec![Err(anyhow!(why))],
+            Ok(scanner) => scanner
+        };
+        scanner.set_timeout(120);
 
-            match scan_result {
-                Err(why) => {
-                    results.push(Err(anyhow!("yara scan error with '{}': {}", file.display(), why)));
-                }
-                Ok(res) => {
-                    results.extend(res.iter().map(|r| {
-                        log::trace!("new yara finding: {} in '{}'",
-                            scanner_result::escape(&r.identifier),
-                            file.display());
-                        Ok(ScannerFinding::Yara(YaraFinding::from(r)))}
-                    ));
-                }
+        for entry in externals.to_hashmap() {
+            if let Err(why) = scanner.define_variable(entry.0, entry.1) {
+                return vec![Err(anyhow!(why))];
+            }
+        }
+
+        let scan_result = match self.buffer.borrow().is_empty() {
+            true => scanner.scan_file(&file),
+            false => scanner.scan_mem(&self.buffer.borrow()).or_else(|e| Err(yara::Error::Yara(e))),
+        };
+
+        match scan_result {
+            Err(why) => {
+                results.push(Err(anyhow!("yara scan error with '{}': {}", file.display(), why)));
+            }
+            Ok(res) => {
+                results.extend(res.iter().map(|r| {
+                    log::trace!("new yara finding: {} in '{}'",
+                        scanner_result::escape(&r.identifier),
+                        file.display());
+                    Ok(ScannerFinding::Yara(YaraFinding::from(r)))}
+                ));
             }
         }
         results
@@ -127,22 +216,30 @@ impl FileScanner for YaraScanner
 
 impl YaraScanner {
     pub fn new<P>(path: P) -> Result<Self> where P: AsRef<Path> {
-        let mut rules = Vec::new();
+        let mut rules_str = Vec::new();
         let metadata = std::fs::metadata(&path)?;
         if metadata.is_file() {
             if Self::points_to_zip_file(&path)? {
-                Self::add_rules_from_zip(&mut rules, &path)?;
+                Self::add_rules_from_zip(&mut rules_str, &path)?;
             } else if Self::points_to_yara_file(&path)? {
-                Self::add_rules_from_yara(&mut rules, path)?;
+                Self::add_rules_from_yara(&mut rules_str, path)?;
             } else {
                 log::warn!("file '{}' is neither a yara nor a zip file; I'll ignore it", path.as_ref().display());
             }
         } else {
-            Self::add_rules_from_directory(&mut rules, path)?;
+            Self::add_rules_from_directory(&mut rules_str, path)?;
+        }
+
+        let mut compiler = yara::Compiler::new()?;
+        for entry in YaraExternals::dummy().to_hashmap() {
+            compiler.define_variable(entry.0, entry.1)?;
+        }
+        for rule in rules_str.into_iter() {
+            compiler = compiler.add_rules_str(&rule)?;
         }
 
         Ok(Self {
-            rules: rules,
+            rules: compiler.compile_rules()?,
             scan_compressed: false,
             magic: magic!().unwrap(),
             buffer: RefCell::new(Vec::with_capacity(1024*1024*128))
@@ -154,45 +251,33 @@ impl YaraScanner {
         self
     }
 
-    #[allow(unused_mut)]
-    pub fn with_buffer_size(mut self, buffer_size: usize) -> Self {
+    pub fn with_buffer_size(self, buffer_size: usize) -> Self {
         self.buffer.replace(Vec::with_capacity(1024*1024*buffer_size));
         self
     }
 
-    fn add_rules_from_yara<P>(rules: &mut Vec<yara::Rules>, path: P) -> Result<()> where P: AsRef<Path> {
+    fn add_rules_from_yara<P>(
+        rules: &mut Vec<String>,
+        path: P) -> Result<()> where P: AsRef<Path> {
         Self::add_rules_from_stream(rules, &path, &mut BufReader::new(File::open(&path)?))
     }
 
-    fn add_rules_from_stream<P, R>(my_rules: &mut Vec<yara::Rules>, path: P, stream: &mut R) -> Result<()> where P: AsRef<Path>, R: std::io::Read {
-        log::info!("parsing yara file: '{}'", path.as_ref().display());
+    fn add_rules_from_stream<P, R>(
+            rules: &mut Vec<String>,
+            path: P, stream: &mut R) -> Result<()> where P: AsRef<Path>, R: std::io::Read {
+        log::trace!("parsing yara file: '{}'", path.as_ref().display());
         let mut yara_content = String::new();
         stream.read_to_string(&mut yara_content)?;
-
-        // FIXME: currently, we use a new compiler for every rules,
-        // because of https://github.com/Hugal31/yara-rust/issues/47
-        match yara::Compiler::new()?.add_rules_str(&yara_content) {
-            Ok(compiler) => {
-                match compiler.compile_rules() {
-                    Ok(rules) => {
-                        my_rules.push(rules);
-                    }
-                    Err(why) => {
-                        log::warn!("yara: compiler error in '{}'", path.as_ref().display());
-                        log::warn!("message was: '{}'", why);
-                    }
-                }
-            }
-            Err(why) => {
-                log::warn!("yara: unable to load content from '{}'", path.as_ref().display());
-                log::warn!("message was: '{}'", why);
-            }
-        }
+        
+        rules.push(yara_content);
         
         Ok(())
     }
 
-    fn add_rules_from_zip<P>(rules: &mut Vec<yara::Rules>, path: P) -> Result<()> where P: AsRef<Path> {
+    fn add_rules_from_zip<P>(
+        rules: &mut Vec<String>,
+        path: P) -> Result<()> where P: AsRef<Path> {
+
         let zip_file = BufReader::new(File::open(&path)?);
         let mut zip = zip::ZipArchive::new(zip_file)?;
         for i in 0..zip.len() {
@@ -220,7 +305,9 @@ impl YaraScanner {
         Ok(())
     }
     
-    fn add_rules_from_directory<P>(rules: &mut Vec<yara::Rules>, path: P) -> Result<()> where P: AsRef<Path> {
+    fn add_rules_from_directory<P>(
+        rules: &mut Vec<String>,
+        path: P) -> Result<()> where P: AsRef<Path> {
         for entry in WalkDir::new(path).into_iter().filter_map(|e| e.ok()) {
             let path = entry.path();
             if Self::points_to_yara_file(&path)? {
